"""Tests for the compute function."""
import datetime
import os
import unittest

import pandas as pd

from timeseriesfeatures.compute import compute
from timeseriesfeatures.feature import Feature, FEATURE_TYPE_LAG, VALUE_TYPE_INT
from timeseriesfeatures.transform import Transform


class TestCompute(unittest.TestCase):

    def setUp(self):
        self.dir = os.path.dirname(__file__)

    def test_compute(self):
        rows = 100
        df = pd.DataFrame(data={
            "feature1": [float(x) for x in range(rows)],
            "feature2": [float(x + 1) for x in range(rows)],
        }, index=[
            datetime.datetime(2022, 1, 1) + datetime.timedelta(x) for x in range(rows)
        ])
        features = compute(df, 30)
        expected_features = [
            Feature(feature_type=FEATURE_TYPE_LAG, columns=["feature1"], value1=VALUE_TYPE_INT, value2=1, transform=str(Transform.NONE)),
            Feature(feature_type=FEATURE_TYPE_LAG, columns=["feature1"], value1=VALUE_TYPE_INT, value2=2, transform=str(Transform.NONE)),
            Feature(feature_type=FEATURE_TYPE_LAG, columns=["feature1"], value1=VALUE_TYPE_INT, value2=3, transform=str(Transform.NONE)),
            Feature(feature_type=FEATURE_TYPE_LAG, columns=["feature1"], value1=VALUE_TYPE_INT, value2=4, transform=str(Transform.NONE)),
            Feature(feature_type=FEATURE_TYPE_LAG, columns=["feature2"], value1=VALUE_TYPE_INT, value2=1, transform=str(Transform.NONE)),
            Feature(feature_type=FEATURE_TYPE_LAG, columns=["feature2"], value1=VALUE_TYPE_INT, value2=2, transform=str(Transform.NONE)),
            Feature(feature_type=FEATURE_TYPE_LAG, columns=["feature2"], value1=VALUE_TYPE_INT, value2=3, transform=str(Transform.NONE)),
            Feature(feature_type=FEATURE_TYPE_LAG, columns=["feature2"], value1=VALUE_TYPE_INT, value2=4, transform=str(Transform.NONE)),
        ]
        self.assertListEqual(features, expected_features)



"""The function for automatically computing features."""

import pandas as pd
from statsmodels.tsa.stattools import acf, pacf  # type: ignore

from .feature import FEATURE_TYPE_LAG, VALUE_TYPE_INT, Feature
from .non_categorical_numeric_columns import \
    find_non_categorical_numeric_columns
from .transform import Transform
from .transforms import TRANSFORMS


def _sort_acf_vals(acf_vals: list[float]) -> list[int]:
    return [
        lag
        for lag, val in sorted(
            [
                (lag, val)
                for lag, val in enumerate(acf_vals)
                if abs(val) > 0.3 and lag != 0
            ],
            key=lambda x: abs(x[1]),
            reverse=True,
        )[:4]
    ]


def compute(
    df: pd.DataFrame,
    max_lag: int,
) -> list[Feature]:
    """Process the dataframe for determining timeseries features."""
    features = []
    columns = find_non_categorical_numeric_columns(df)
    for column in columns:
        for transform in Transform:
            transformed_series = TRANSFORMS[transform](df[column])
            for lag in _sort_acf_vals(
                acf(transformed_series, nlags=max_lag, fft=False)  # type: ignore
            ) + _sort_acf_vals(pacf(transformed_series, nlags=max_lag)):  # type: ignore
                new_feature = Feature(
                    feature_type=FEATURE_TYPE_LAG,
                    columns=[column],
                    value1=VALUE_TYPE_INT,
                    value2=lag,
                    transform=str(Transform.NONE),
                )
                if new_feature in features:
                    continue
                features.append(new_feature)
    return features
